@@ -7,6 +7,7 @@ mod walletdata;
 use std::{collections::BTreeMap, ffi::CString, net::SocketAddr, path::PathBuf, sync::Arc};
 
 use anyhow::Context;
+use base32::Alphabet;
 use http_types::headers::HeaderValue;
 use multi::MultiWallet;
 use serde::Deserialize;
@@ -152,6 +153,8 @@ fn main() -> anyhow::Result<()> {
         app.at("/wallets/:name").put(create_wallet);
         app.at("/wallets/:name/lock").post(lock_wallet);
         app.at("/wallets/:name/unlock").post(unlock_wallet);
+        app.at("/wallets/:name/export-sk")
+            .post(export_sk_from_wallet);
         app.at("/wallets/:name/check").put(check_wallet);
         app.at("/wallets/:name/coins").get(dump_coins);
         app.at("/wallets/:name/prepare-tx").post(prepare_tx);
@@ -234,10 +237,23 @@ async fn create_wallet(mut req: Request<Arc<AppState>>) -> tide::Result<Body> {
     struct Query {
         testnet: bool,
         password: Option<String>,
+        secret: Option<String>,
     }
     let query: Query = req.body_json().await?;
     let wallet_name = req.param("name").map(|v| v.to_string())?;
-    let (_, sk) = tmelcrypt::ed25519_keygen();
+    let sk = if let Some(secret) = query.secret {
+        // We must reconstruct the secret key using the ed25519-dalek library
+        let secret =
+            base32::decode(Alphabet::Crockford, &secret).context("cannot decode secret key")?;
+        let secret = ed25519_dalek::SecretKey::from_bytes(&secret)?;
+        let public: ed25519_dalek::PublicKey = (&secret).into();
+        let mut vv = [0u8; 64];
+        vv[0..32].copy_from_slice(&secret.to_bytes());
+        vv[32..].copy_from_slice(&public.to_bytes());
+        Ed25519SK(vv)
+    } else {
+        tmelcrypt::ed25519_keygen().1
+    };
     req.state()
         .create_wallet(
             &wallet_name,
@@ -314,6 +330,23 @@ async fn unlock_wallet(mut req: Request<Arc<AppState>>) -> tide::Result<Body> {
     Ok("".into())
 }
 
+async fn export_sk_from_wallet(mut req: Request<Arc<AppState>>) -> tide::Result<Body> {
+    check_auth(&req)?;
+    #[derive(Deserialize)]
+    struct Req {
+        password: Option<String>,
+    }
+    let wallet_name = req.param("name").map(|v| v.to_string())?;
+    let request: Req = req.body_json().await?;
+    // attempt to unlock
+    let secret = req
+        .state()
+        .get_secret_key(&wallet_name, request.password)
+        .context("incorrect password")
+        .map_err(to_forbidden)?;
+    Ok(base32::encode(Alphabet::Crockford, &secret.0[..32]).into())
+}
+
 async fn prepare_stake_tx(req: Request<Arc<AppState>>) -> tide::Result<Body> {
     check_auth(&req)?;
     todo!()
@@ -576,10 +609,10 @@ fn to_badgateway<E: Into<anyhow::Error> + Send + 'static + Sync + Debug>(e: E) -
     tide::Error::new(StatusCode::BadGateway, e)
 }
 
-fn notfound_with(s: String) -> tide::Error {
-    tide::Error::new(StatusCode::NotFound, anyhow::anyhow!("{s}"))
-}
+// fn notfound_with(s: String) -> tide::Error {
+//     tide::Error::new(StatusCode::NotFound, anyhow::anyhow!("{s}"))
+// }
 
-fn wallet_notfound() -> tide::Error {
-    notfound_with("wallet not found".into())
-}
+// fn wallet_notfound() -> tide::Error {
+//     notfound_with("wallet not found".into())
+// }
